@@ -1,6 +1,6 @@
 /*
 #   live_aac_encoder.c: encode using libavformat
-#   Copyright (C) 2015 Stephen Fairchild (s-fairchild@users.sourceforge.net)
+#   Copyright (C) 2015-2022 Stephen Fairchild (s-fairchild@users.sf.net)
 #
 #   This program is free software: you can redistribute it and/or modify
 #   it under the terms of the GNU General Public License as published by
@@ -31,6 +31,7 @@
 #include <libavutil/opt.h>
 #include <libavutil/mathematics.h>
 #include <libavutil/timestamp.h>
+#include <libavcodec/avcodec.h>
 #include <libavformat/avformat.h>
 #include <libswscale/swscale.h>
 #include <libswresample/swresample.h>
@@ -57,7 +58,8 @@ typedef struct State {
     char *metadata;
     uint8_t *buf;
     size_t buf_size;
-    int sri;
+    int sfi;
+    AVPacket *pkt;
 } State;
 
 
@@ -70,13 +72,13 @@ static void avcodec_safe_close(AVCodecContext **c)
 }
 
 
-static AVCodec *add_stream(State *self, enum AVCodecID codec_id,
+static const AVCodec *add_stream(State *self, enum AVCodecID codec_id,
                            int profile, int br, int sr, int ch)
 {
-    AVCodec *codec;
     AVCodecContext *c;
+    const AVCodec *codec = avcodec_find_encoder(codec_id);
 
-    if (!(codec = avcodec_find_encoder(codec_id))) {
+    if (!codec) {
         fprintf(stderr, "could not find encoder for '%s'\n",
                                                 avcodec_get_name(codec_id));
         return NULL;
@@ -97,12 +99,35 @@ static AVCodec *add_stream(State *self, enum AVCodecID codec_id,
         fprintf(stderr, "failed to allocate the codec context\n");
         return NULL;
     }
-    c->sample_fmt  = codec->sample_fmts ?
-                                    codec->sample_fmts[0] : AV_SAMPLE_FMT_FLTP;
+
+    if (codec->sample_fmts) {
+        c->sample_fmt = codec->sample_fmts[0];
+        for (int i = 0; codec->sample_fmts[i]; i++) {
+            if (codec->sample_fmts[i] == AV_SAMPLE_FMT_FLTP) {
+                c->sample_fmt = AV_SAMPLE_FMT_FLTP;
+                break;
+            }
+            if (codec->sample_fmts[i] == AV_SAMPLE_FMT_FLT) {
+                c->sample_fmt = AV_SAMPLE_FMT_FLT;
+                break;
+            }
+        }
+    } else
+        c->sample_fmt = AV_SAMPLE_FMT_FLTP;
+
     c->bit_rate    = br;
     c->sample_rate = sr;
-    c->channels    = ch;
-    c->channel_layout = (ch == 2) ? AV_CH_LAYOUT_STEREO : AV_CH_LAYOUT_MONO;
+    c->ch_layout.order = AV_CHANNEL_ORDER_NATIVE;
+    c->ch_layout.nb_channels = ch;
+    switch (ch) {
+        case 1:  c->ch_layout.u.mask = AV_CH_LAYOUT_MONO;
+                 break;
+        case 2:  c->ch_layout.u.mask = AV_CH_LAYOUT_STEREO;
+                 break;
+        default: fprintf(stderr, "unsupported number of channels: %d\n", ch);
+                 return NULL;
+    }
+
     c->profile = profile;
     self->st->id = 0;
     self->st->time_base = (AVRational){ 1, sr };
@@ -119,7 +144,7 @@ static AVCodec *add_stream(State *self, enum AVCodecID codec_id,
 
 
 static AVFrame *alloc_audio_frame(enum AVSampleFormat sample_fmt,
-                    uint64_t channel_layout, int sample_rate, int nb_samples)
+                AVChannelLayout *ch_layout, int sample_rate, int nb_samples)
 {
     AVFrame *frame = av_frame_alloc();
 
@@ -129,7 +154,7 @@ static AVFrame *alloc_audio_frame(enum AVSampleFormat sample_fmt,
     }
 
     frame->format = sample_fmt;
-    frame->channel_layout = channel_layout;
+    frame->ch_layout = *ch_layout;
     frame->sample_rate = sample_rate;
     frame->nb_samples = nb_samples;
 
@@ -143,12 +168,12 @@ static AVFrame *alloc_audio_frame(enum AVSampleFormat sample_fmt,
 }
 
 
-static int open_stream(State *self, AVCodec *codec)
+static int open_stream(State *self, const AVCodec *codec)
 {
     AVCodecContext *c;
     int nb_samples;
     int ret;
-
+    int nb_channels;
     c = self->c;
 
     while (pthread_mutex_trylock(&g.avc_mutex))
@@ -171,9 +196,9 @@ static int open_stream(State *self, AVCodec *codec)
         nb_samples = 10000;
 #endif
 
-    self->frame     = alloc_audio_frame(c->sample_fmt, c->channel_layout,
+    self->frame     = alloc_audio_frame(c->sample_fmt, &c->ch_layout,
                                        c->sample_rate, nb_samples);
-    self->tmp_frame = alloc_audio_frame(AV_SAMPLE_FMT_FLTP, c->channel_layout,
+    self->tmp_frame = alloc_audio_frame(AV_SAMPLE_FMT_FLTP, &c->ch_layout,
                                        c->sample_rate, nb_samples);
 
     if (!(self->swr_ctx = swr_alloc())) {
@@ -182,11 +207,14 @@ static int open_stream(State *self, AVCodec *codec)
         return 0;
     }
 
-    av_opt_set_int       (self->swr_ctx, "in_channel_count",   c->channels,       0);
+    nb_channels = c->ch_layout.nb_channels;
+    av_opt_set_int       (self->swr_ctx, "in_channel_count",   nb_channels,       0);
     av_opt_set_int       (self->swr_ctx, "in_sample_rate",     c->sample_rate,    0);
+    av_opt_set_chlayout  (self->swr_ctx, "in_chlayout",        &c->ch_layout, 0);
     av_opt_set_sample_fmt(self->swr_ctx, "in_sample_fmt",      AV_SAMPLE_FMT_FLTP, 0);
-    av_opt_set_int       (self->swr_ctx, "out_channel_count",  c->channels,       0);
+    av_opt_set_int       (self->swr_ctx, "out_channel_count",  nb_channels,       0);
     av_opt_set_int       (self->swr_ctx, "out_sample_rate",    c->sample_rate,    0);
+    av_opt_set_chlayout  (self->swr_ctx, "out_chlayout",       &c->ch_layout, 0);
     av_opt_set_sample_fmt(self->swr_ctx, "out_sample_fmt",     c->sample_fmt,     0);
 
     if ((ret = swr_init(self->swr_ctx)) < 0) {
@@ -224,13 +252,17 @@ static int write_audio_frame(struct encoder *encoder, int final)
 {
     State *self = encoder->encoder_private;
     AVCodecContext *c;
-    AVPacket pkt = { 0 };
     AVFrame *frame;
     int ret;
     int got_packet;
     int dst_nb_samples;
 
-    av_init_packet(&pkt);
+    if (self->pkt)
+        av_packet_unref(self->pkt);
+    if (!(self->pkt = av_packet_alloc())) {
+        fprintf(stderr, "av_packet_init failed\n");
+        return -1;
+    }
     c = self->c;
     if (final)
         frame = NULL;
@@ -259,29 +291,16 @@ static int write_audio_frame(struct encoder *encoder, int final)
         self->samples_count += dst_nb_samples;
     }
 
-
-#ifdef HAVE_AVCODEC_RECEIVE_PACKET
-        if ((ret = avcodec_send_frame(c, frame)) < 0) {
-            fprintf(stderr, "error encoding audio frame: %s\n", av_err2str(ret));
-            return -1;
-        }
-
-        for (got_packet = 0; !avcodec_receive_packet(c, &pkt); got_packet = 1)
-            if ((ret = av_write_frame(self->oc, &pkt)) < 0) {
-                fprintf(stderr, "error while writing audio frame: %s\n", av_err2str(ret));
-                return -1;
-            }
-#else
-    if ((ret = avcodec_encode_audio2(c, &pkt, frame, &got_packet)) < 0) {
+    if ((ret = avcodec_send_frame(c, frame)) < 0) {
         fprintf(stderr, "error encoding audio frame: %s\n", av_err2str(ret));
         return -1;
     }
 
-    if (got_packet && (ret = av_write_frame(self->oc, &pkt)) < 0) {
-        fprintf(stderr, "error while writing audio frame: %s\n", av_err2str(ret));
-        return -1;
-    }
-#endif
+    for (got_packet = 0; !avcodec_receive_packet(c, self->pkt); got_packet = 1)
+        if ((ret = av_write_frame(self->oc, self->pkt)) < 0) {
+            fprintf(stderr, "error while writing audio frame: %s\n", av_err2str(ret));
+            return -1;
+        }
 
     if (final)
         return 1;
@@ -295,6 +314,8 @@ static int write_audio_frame(struct encoder *encoder, int final)
 
 static void close_stream(State *self)
 {
+    if (self->pkt)
+        av_packet_unref(self->pkt);
     avcodec_safe_close(&self->c);
     av_frame_free(&self->frame);
     av_frame_free(&self->tmp_frame);
@@ -324,7 +345,7 @@ static void packetize_metadata(struct encoder *e, State * const s)
 static int write_packet(struct encoder *encoder, uint8_t *buf, int buf_size, enum packet_flags pf)
 {
     State *self = encoder->encoder_private;
-    struct encoder_op_packet packet;
+    struct encoder_op_packet_rb_feed packet;
 
     packet.header.bit_rate = encoder->bitrate;
     packet.header.sample_rate = encoder->target_samplerate;
@@ -340,11 +361,18 @@ static int write_packet(struct encoder *encoder, uint8_t *buf, int buf_size, enu
     return 1;
 }
 
-static int write_packet_wrapper(void *opaque, uint8_t *buf, int buf_size)
+static int write_packet_with_adts(void *opaque, const uint8_t *buf, int buf_size)
 {
     struct encoder *encoder = (struct encoder *)opaque;
     State *self = encoder->encoder_private;
 
+    if (buf_size > 16376) {
+        // adts header frame size limited to 13 bits
+        fprintf(stderr, "error: frame size too large for adts header\n");
+        buf_size = 16376;
+        encoder->encoder_state = ES_STOPPING;
+    }
+
     if (7 + buf_size > self->buf_size)
         if (!(self->buf = realloc(self->buf, self->buf_size = 7 + buf_size))) {
             fprintf(stderr, "malloc failure\n");
@@ -355,12 +383,12 @@ static int write_packet_wrapper(void *opaque, uint8_t *buf, int buf_size)
         uint8_t *b = self->buf;
         *b++ = 0xff;    // syncword
         *b++ = 0xf1;    // syncword, mpeg 4, layer 0, unprotected
-        *b++ = 0x40 | self->sri;
+        *b++ = 0x40 | (self->sfi << 2); // low complexity, sample frequency index, private unset, in-band channel config
         int s = 7 + buf_size;
-        *b++ = encoder->n_channels << 6 | s >> 11;
-        *b++ = (s >> 3) & 0xFF;
-        *b++ = ((s << 5) & 0xFF) | 0x1f;
-        *b++ = 0xFC;
+        *b++ = encoder->n_channels << 6 | s >> 11; // in-band channel config, originality false, home false, copyright id bits unset, frame length
+        *b++ = (s >> 3) & 0xFF;             // frame length
+        *b++ = ((s << 5) & 0xFF) | 0x1f;    // frame length, buffer fullness all 1's
+        *b++ = 0xFC;                        // buffer fullness all 1's, 1 data block per frame
     }
 
     memcpy(self->buf + 7, buf, buf_size);
@@ -380,7 +408,6 @@ static int write_header(struct encoder *encoder)
     return ret;
 }
 
-
 static int write_trailer(struct encoder *encoder)
 {
     State *self = encoder->encoder_private;
@@ -393,12 +420,22 @@ static int write_trailer(struct encoder *encoder)
     return ret;
 }
 
+static int sr2adts_sfi(long sr)
+{
+    const long table[] = {96000, 88200, 64000, 48000, 44100, 32000, 24000,
+                          22050, 16000, 12000, 11025, 8000, 7350, 0};
+
+    for (int i = 0; table[i]; i++) {
+        if (table[i] == sr)
+            return i;
+    }
+    return -1;
+}
 
 static int setup(struct encoder *encoder)
 {
     State *self = encoder->encoder_private;
     size_t avio_ctx_buffer_size = 4096;
-    AVCodec *codec;
     uint8_t *avio_ctx_buffer;
     enum AVCodecID codec_id;
     int profile;
@@ -433,16 +470,23 @@ static int setup(struct encoder *encoder)
         goto fail2;
     }
 
+// ffmpeg < 7 wants non const pointer in write packet signature
+#pragma GCC diagnostic push
+#pragma GCC diagnostic ignored "-Wincompatible-pointer-types"
     if (!(self->avio_ctx = avio_alloc_context(avio_ctx_buffer,
-            avio_ctx_buffer_size, 1, encoder, NULL, &write_packet_wrapper, NULL))) {
+            avio_ctx_buffer_size, 1, encoder, NULL, &write_packet_with_adts, NULL))) {
         fprintf(stderr, "avio_alloc_context failed\n");
         goto fail3;
     }
+#pragma GCC diagnostic pop
 
     self->oc->pb = self->avio_ctx;
 
-    if (!(codec = add_stream(self, codec_id, profile, encoder->bitrate,
-                        encoder->target_samplerate, encoder->n_channels))) {
+    const AVCodec *codec = add_stream(self, codec_id, profile,
+                                      encoder->bitrate,
+                                      encoder->target_samplerate,
+                                      encoder->n_channels);
+    if (!codec) {
         fprintf(stderr, "failed to add stream\n");
         goto fail4;
     }
@@ -455,52 +499,10 @@ static int setup(struct encoder *encoder)
     if (write_header(encoder) < 0)
         goto fail5;
 
-    int sri;
-    switch (encoder->target_samplerate) {
-        case 96000:
-            sri = 0;
-            break;
-        case 88200:
-            sri = 1;
-            break;
-        case 64000:
-            sri = 2;
-            break;
-        case 48000:
-            sri = 3;
-            break;
-        case 44100:
-            sri = 4;
-            break;
-        case 32000:
-            sri = 5;
-            break;
-        case 24000:
-            sri = 6;
-            break;
-        case 22050:
-            sri = 7;
-            break;
-        case 16000:
-            sri = 8;
-            break;
-        case 12000:
-            sri = 9;
-            break;
-        case 11025:
-            sri = 10;
-            break;
-        case 8000:
-            sri = 11;
-            break;
-        case 7350:
-            sri = 12;
-            break;
-        default:
-            fprintf(stderr, "live_aac_encoder.c: bad sample rate index\n");
-            goto fail5;
+    if ((self->sfi = sr2adts_sfi(encoder->target_samplerate)) < 0) {
+        fprintf(stderr, "live_aac_encoder.c: unsupported sample rate\n");
+        goto fail5;
     }
-    self->sri = sri << 2;
 
     return SUCCEEDED;
 
