@@ -38,7 +38,7 @@ static int threads_up;
 static void threads_init(struct threads_info *ti)
     {
     int i;
-    
+
     ti->n_encoders = atoi(getenv("num_encoders"));
     ti->n_streamers = atoi(getenv("num_streamers"));
     ti->n_recorders = atoi(getenv("num_recorders"));
@@ -80,7 +80,7 @@ static void threads_init(struct threads_info *ti)
 static void threads_shutdown(struct threads_info *ti)
     {
     int i;
-    
+
     if (threads_up)
         {
         for (i = 0; i < ti->n_recorders; i++)
@@ -109,7 +109,7 @@ static int get_report(struct threads_info *ti, struct universal_vars *uv, void *
         {
         if (uv->tab >= 0 && uv->tab < ti->n_recorders)
             return recorder_make_report(ti->recorder[uv->tab]);
-        fprintf(stderr, "get_report: recorder %s does not exist\n", uv->tab_id); 
+        fprintf(stderr, "get_report: recorder %s does not exist\n", uv->tab_id);
         return FAILED;
         }
     if (!strcmp(uv->dev_type, "encoder"))
@@ -195,6 +195,8 @@ static struct kvpdict kvpdict[] = {
     { "tab_id",   &uv.tab_id, NULL},
     { NULL, NULL, NULL } };
 
+static GHashTable *ht;
+
 static struct commandmap commandmap[] = {
     { "jack_samplerate_request", audio_feed_jack_samplerate_request, NULL },
     { "encoder_lame_availability", encoder_init_lame, NULL},
@@ -211,12 +213,13 @@ static struct commandmap commandmap[] = {
     { "server_connect", streamer_connect, &sv },
     { "server_disconnect", streamer_disconnect, NULL },
     { "initiate_fade", encoder_initiate_fade, NULL },
-    { NULL, NULL, NULL } }; 
+    { NULL, NULL, NULL } };
 
 static void sourceclient_cleanup()
     {
     threads_shutdown(&ti);
     kvp_free_dict(kvpdict);
+    g_hash_table_unref(ht);
     }
 
 void sourceclient_init()
@@ -226,14 +229,18 @@ void sourceclient_init()
     setlocale(LC_ALL, "C");
 
     srand(time(NULL));
-    
+
+    if (!(ht = kvp_ht_init(kvpdict))) {
+        fprintf(stderr, "failed to initialize hash table\n");
+        exit(5);
+    }
     threads_init(&ti);
     atexit(sourceclient_cleanup);
     }
 
 int sourceclient_main()
     {
-    if (!kvp_parse(kvpdict, g.in))
+    if (!kvp_parse(ht, g.in))
         return FALSE;
 
     if (uv.command && command_parse(commandmap, &ti, &uv))
@@ -248,6 +255,6 @@ int sourceclient_main()
         free(uv.command);
         uv.command = NULL;
         }
-        
+
     return TRUE;
     }
