@@ -1,6 +1,6 @@
 /*
 #   streamer.c: the streaming part of the streaming module of idjc
-#   Copyright (C) 2007 Stephen Fairchild (s-fairchild@users.sourceforge.net)
+#   Copyright (C) 2007-2024 Stephen Fairchild (s-fairchild@users.sourceforge.net)
 #
 #   This program is free software: you can redistribute it and/or modify
 #   it under the terms of the GNU General Public License as published by
@@ -24,15 +24,16 @@
 #include <stdlib.h>
 #include <string.h>
 #include <pthread.h>
+#ifdef HAVE_REAL_LIBSHOUTIDJC
 #include <shoutidjc/shout.h>
+#else
+#include <shout/shout.h>
+#endif /* HAVE_REAL_LIBSHOUTIDJC */
+
 #include "sourceclient.h"
 #include "sig.h"
 #include "main.h"
 
-/* other versions of libshout define SHOUT_FORMAT_VORBIS instead */
-#ifndef SHOUT_FORMAT_OGG
-#define SHOUT_FORMAT_OGG SHOUT_FORMAT_VORBIS
-#endif  /*SHOUT_FORMAT_OGG*/
 
 /* the number of seconds of audio to stockpile before packet dumping takes place */
 static const int shout_buffer_seconds = 9;
@@ -44,7 +45,8 @@ static void *streamer_main(void *args)
     struct encoder_op_packet *packet;
     size_t data_size;
     unsigned connect_time = 0;
-    
+    int try_count = 10;
+
     sig_mask_thread();
     while (!self->thread_terminate_f)
         {
@@ -61,6 +63,8 @@ static void *streamer_main(void *args)
             case SM_CONNECTING:
                 switch(self->shout_status)
                     {
+                    case SHOUTERR_RETRY:
+                        break;
                     case SHOUTERR_BUSY:
                         self->shout_status = shout_get_connected(self->shout);
 
@@ -82,8 +86,13 @@ static void *streamer_main(void *args)
                 break;
             case SM_CONNECTED:
                 /* check the connection is still on */
+
                 if ((self->shout_status = shout_get_connected(self->shout)) != SHOUTERR_CONNECTED)
                     {
+                    if (self->shout_status == SHOUTERR_RETRY && try_count-- > 0) {
+                        fprintf(stderr, "retry\n");
+                        break;
+                    }
                     fprintf(stderr, "streamer_main: shout_get_error reports %ld %s\n", self->shout_status, shout_get_error(self->shout));
                     self->stream_mode = SM_DISCONNECTING;
                     }
@@ -101,7 +110,7 @@ static void *streamer_main(void *args)
                         if (packet->header.flags & PF_INITIAL)
                             {
                             int br = packet->header.bit_rate;
-                            
+
                             /* determine how much audio to hold in the send buffer */
                             self->max_shout_queue = (shout_buffer_seconds * ((br > 1000) ? br / 1000 : br)) << 7;
                             }
@@ -114,11 +123,12 @@ static void *streamer_main(void *args)
                                 data_size = 0;
                                 fprintf(stderr, "streamer_main: **** packet dumped due to buffer being full ****\n");
                                 }
-#if 1                           
+#if 1
                             switch(shout_send(self->shout, packet->data, data_size))
                                 {
                                 case SHOUTERR_SUCCESS:
                                 case SHOUTERR_BUSY:
+                                case SHOUTERR_RETRY:
                                     break;
                                 default:
                                     fprintf(stderr, "streamer_main: failed writing to stream, shout_get_error reports: %s\n", shout_get_error(self->shout));
@@ -144,16 +154,19 @@ static void *streamer_main(void *args)
                         {
                         *strpbrk(packet->data, "\n") = '\0';
                         fprintf(stderr, "streamer_main: packet is metadata: %s\n", (char *)packet->data);
-                        shout_metadata_add(self->shout_meta, "song", packet->data);
-                        switch (shout_set_metadata(self->shout, self->shout_meta))
-                            {
-                            case SHOUTERR_SUCCESS:
-                            case SHOUTERR_BUSY:
-                                break;
-                            default:
-                                fprintf(stderr, "streamer_main: failed writing metadata to stream, shout_get_error reports: %s\n", shout_get_error(self->shout));
-                                self->stream_mode = SM_DISCONNECTING;
+                        if (shout_metadata_add(self->shout_meta, "song", packet->data) == SHOUTERR_SUCCESS) {
+                            switch (shout_set_metadata_utf8(self->shout, self->shout_meta))
+                                {
+                                case SHOUTERR_SUCCESS:
+                                case SHOUTERR_BUSY:
+                                    break;
+                                case SHOUTERR_RETRY:
+                                default:
+                                    fprintf(stderr, "streamer_main: failed writing metadata to stream, shout_get_error reports: %s\n", shout_get_error(self->shout));
+                                }
                             }
+                        else
+                            fprintf(stderr, "streamer_main: shout_metadata_add failed\n");
                         }
                     encoder_client_free_packet(packet);
                     }
@@ -200,8 +213,9 @@ int streamer_connect(struct threads_info *ti, struct universal_vars *uv, void *o
     struct streamer *self = ti->streamer[uv->tab];
     int protocol, data_format = -1, tls;
     char channels[2];
-    char bitrate[4];
+    char bitrate[6];
     char samplerate[6];
+    int try_count = 10;
 
     void sce(char *parameter)    /* stream connect error */
         {
@@ -221,7 +235,7 @@ int streamer_connect(struct threads_info *ti, struct universal_vars *uv, void *o
         }
     else
         {
-        const struct encoder_data_format *df = &self->encoder_op->encoder->data_format;            
+        const struct encoder_data_format *df = &self->encoder_op->encoder->data_format;
         int failed = FALSE;
 
         switch (df->family) {
@@ -236,32 +250,42 @@ int streamer_connect(struct threads_info *ti, struct universal_vars *uv, void *o
                         data_format = SHOUT_FORMAT_MP3;
                         break;
                     case ENCODER_CODEC_AAC:
+#ifdef SHOUT_FORMAT_AAC
                         data_format = SHOUT_FORMAT_AAC;
+#else
+/* user interface bars this execution path */
+                        failed = TRUE;
+#endif /* SHOUT_FORMAT_AAC */
                         break;
                     case ENCODER_CODEC_AACPLUSV2:
+#ifdef SHOUT_FORMAT_AACPLUS
                         data_format = SHOUT_FORMAT_AACPLUS;
+#else
+/* user interface bars this exectuion path */
+                        failed = TRUE;
+#endif /* SHOUT_FORMAT_AACPLUS */
                         break;
                     case ENCODER_CODEC_UNHANDLED:
                     default:
                         failed = TRUE;
                 }
                 break;
-                
+
             case ENCODER_FAMILY_WEBM:
                 switch (df->codec) {
                     case ENCODER_CODEC_VORBIS:
                     case ENCODER_CODEC_OPUS:
-                        data_format = SHOUT_FORMAT_WEBMAUDIO;
+                        data_format = SHOUT_FORMAT_WEBM;
                         break;
                     default:
                         failed = TRUE;
                 }
                 break;
-                
+
             case ENCODER_FAMILY_UNHANDLED:
                 failed = TRUE;
             }
-            
+
         if (failed)
             {
             fprintf(stderr, "streamer_start: unhandled encoder data format\n");
@@ -269,7 +293,7 @@ int streamer_connect(struct threads_info *ti, struct universal_vars *uv, void *o
             return FAILED;
             }
         }
-        
+
     if (!strcmp(sv->server_type, "Shoutcast"))
         protocol = SHOUT_PROTOCOL_ICY;
     else if (!strcmp(sv->server_type, "Icecast 2"))
@@ -298,9 +322,9 @@ int streamer_connect(struct threads_info *ti, struct universal_vars *uv, void *o
         sce("protocol");
         goto error;
         }
-    if (shout_set_format(self->shout, data_format) != SHOUTERR_SUCCESS)
+    if (shout_set_content_format(self->shout, data_format, SHOUT_USAGE_AUDIO, NULL) != SHOUTERR_SUCCESS)
         {
-        sce("format");
+        sce("content_format");
         goto error;
         }
     if (shout_set_host(self->shout, sv->host) != SHOUTERR_SUCCESS)
@@ -415,14 +439,14 @@ int streamer_connect(struct threads_info *ti, struct universal_vars *uv, void *o
         goto error;
         }
     #endif
-        
+
     snprintf(channels,   sizeof channels  , "%d",  self->encoder_op->encoder->n_channels);
     {
         int br = self->encoder_op->encoder->bitrate;
-        snprintf(bitrate, sizeof bitrate   , "%d",  ((br < 1000) ? br : br/1000));
+        snprintf(bitrate, sizeof bitrate   , "%hd",  (int16_t)((br < 1000) ? br : br/1000));
     }
     snprintf(samplerate, sizeof samplerate, "%ld", self->encoder_op->encoder->target_samplerate);
-        
+
     if (shout_set_audio_info(self->shout, SHOUT_AI_BITRATE, bitrate) != SHOUTERR_SUCCESS)
         {
         sce("set_audio_info bitrate");
@@ -438,14 +462,21 @@ int streamer_connect(struct threads_info *ti, struct universal_vars *uv, void *o
         sce("set_audio_info channels");
         goto error;
         }
-        
+
     if (shout_set_nonblocking(self->shout, 1) != SHOUTERR_SUCCESS)
         {
         sce("non-blocking");
         goto error;
         }
+    retry:
+    fprintf(stderr, "calling shout_open: remaining tries=%d\n", try_count--);
     switch(self->shout_status = shout_open(self->shout))
         {
+        case SHOUTERR_RETRY:
+            while (try_count > 0) {
+                goto retry;
+            }
+            break;
         case SHOUTERR_SUCCESS:
             self->shout_status = SHOUTERR_CONNECTED;
         case SHOUTERR_BUSY:
@@ -485,14 +516,23 @@ int streamer_disconnect(struct threads_info *ti, struct universal_vars *uv, void
 
 void shout_initialiser()
     {
+    int major, minor, patch;
+#ifdef HAVE_REAL_LIBSHOUTIDJC
+    const char *libname = "libshout-idjc";
+#else
+    const char *libname = "libshout";
+#endif /* HAVE_REAL_LIBSHOUTIDJC */
+
     shout_init();
+    shout_version(&major, &minor, &patch);
+    fprintf(stderr, "%s version %d.%d.%d\n", libname, major, minor, patch);
     }
 
 struct streamer *streamer_init(struct threads_info *ti, int numeric_id)
     {
     struct streamer *self;
     static pthread_once_t once_control = PTHREAD_ONCE_INIT;
-    
+
     pthread_once(&once_control, shout_initialiser);
     if (!(self = calloc(1, sizeof (struct streamer))))
         {
