@@ -21,13 +21,15 @@
 #define XLPLAYER_H
 
 #include "../config.h"
-#include <jack/jack.h>
-#include <jack/ringbuffer.h>
+
+#include <signal.h>
 #include <pthread.h>
 #include <stdlib.h>
+
+#include <jack/jack.h>
+#include <jack/ringbuffer.h>
 #include <samplerate.h>
 #include <sndfile.h>
-#include <signal.h>
 
 #ifdef HAVE_FLAC
 #include <FLAC/all.h>
@@ -83,7 +85,12 @@ struct xlplayer
     int pause;                          /* flag controlling the player paused state */
     int write_deferred;                 /* suppress further generation of audio data */
     u_int64_t samples_written;          /* number of samples written to the ringbuffer */
-    int32_t play_progress_ms;           /* the playback progress in milliseconds */
+    long play_progress_ms;              /* the playback progress in milliseconds */
+    int have_clock;                     /* true if we can get clock resolution for a particular clock */
+    struct timespec clockres;           /* resolution of the speedup clock */
+    struct timespec prevtime;           /* time since last speedup clock read */
+    long prevprogress;                  /* previous play_progress_ms value */
+    double speedup;                     /* how fast the player is playing for freewheel mode */
     char *playername;                   /* the name of this player e.g. "left", "right" etc. */
     enum playmode_t playmode;           /* indicates the player mode or state */
     enum command_t command;             /* the command mode */
@@ -97,7 +104,7 @@ struct xlplayer
     u_int32_t sleep_samples;            /* used to count off when it is appropriate to call sleep */
     SRC_STATE *src_state;               /* used by resampler */
     SRC_DATA src_data;
-    int rsqual;                         /* resample quality */   
+    int rsqual;                         /* resample quality */
     int noflush;                        /* suppresses ringbuffer flushes for gapless playback */
     int *jack_shutdown_f;               /* inidcator that jack has shut down */
     volatile sig_atomic_t watchdog_timer;
@@ -123,19 +130,19 @@ struct xlplayer
     int usedelay;                       /* client to delay dynamic metadata display */
     float silence;                      /* the number of seconds of silence */
     int samples_cutoff;                 /* audio cutoff imminent when fewer than this value samples remain */
-    
+
     int use_sv;                         /* speed variance version of read function will be used */
-    
+
     float *lcb;                         /* left channel buffer */
     float *rcb;                         /* right channel buffer */
     float *lcfb;                        /* left channel fade buffer */
     float *rcfb;                        /* right channel fade buffer */
-    
+
     float *lcp, *rcp, *lcfp, *rcfp;     /* pointers into the above buffers */
-    
+
     float ls, rs;                       /* the current audio sample stereo pair -- prior to gain adjustment */
     float peak;                         /* peak = MAX(peak, MAX(ABS(ls), ABS(rs))) */
-    
+
     struct smoothing_mute mute_aud;
     struct smoothing_mute mute_str;
     struct smoothing_volume volume;
@@ -144,7 +151,7 @@ struct xlplayer
     int cf_aud;                         /* apply crossfade on dj audio */
     float ls_aud, ls_str;               /* the gain adjusted audio samples */
     float rs_aud, rs_str;
-    uint32_t id;                        /* player identity e.g. player 3 = 1 << 3 */
+    long id;                            /* player identity e.g. player 3 = 1 << 3 */
     pthread_mutex_t command_mutex;      /* lock for command varaible change */
     pthread_cond_t command_cv;          /* used to wake up idle worker thread */
     };
@@ -167,7 +174,7 @@ int xlplayer_playmany(struct xlplayer *self, char *playlist, int loop_f);
 /* xlplayer_play_noflush: starts the player without flushing out old data from the ringbuffer */
 int xlplayer_play_noflush(struct xlplayer *self, char *pathname, int seek_s, int size, float gain_db, int id);
 
-/* xlplayer_cancelplaynext: cancels the automatic playing of the next track 
+/* xlplayer_cancelplaynext: cancels the automatic playing of the next track
 * the current track is allowed to continue playing */
 void xlplayer_cancelplaynext(struct xlplayer *self);
 
