@@ -1,6 +1,6 @@
 """The profile management dialog."""
 
-#   Copyright (C) 2011, 2016 Stephen Fairchild
+#   Copyright (C) 2011, 2020 Stephen Fairchild
 #   (s-fairchild@users.sourceforge.net)
 #   This program is free software: you can redistribute it and/or modify
 #   it under the terms of the GNU General Public License as published by
@@ -16,83 +16,71 @@
 #   along with this program in the file entitled COPYING.
 #   If not, see <http://www.gnu.org/licenses/>.
 
-from __future__ import print_function
-
 __all__ = ["ProfileDialog"]
 
 import atexit
+import gettext
 
 # This is and needs to remain the initial gtk import point.
-import glib
-import gobject
-import gtk
-import pango
+from gi.repository import Gtk
+from gi.repository import GLib
+from gi.repository import GObject
+from gi.repository import Pango
+from gi.repository import GdkPixbuf
+from gi.repository import Gdk
+from gi.repository import Gio
 
 from idjc import PGlobs, FGlobs
 from idjc.prelims import MAX_PROFILE_LENGTH, profile_name_valid, default
 from ..utils import Singleton
-
-gtk.gdk.threads_init()
-gtk.gdk.threads_enter()
-atexit.register(gtk.gdk.threads_leave)
-
+from ..utils import cmp
 from ..gtkstuff import ConfirmationDialog
 from ..gtkstuff import ErrorMessageDialog
 from ..gtkstuff import CellRendererLED
 from ..gtkstuff import CellRendererTime
-from ..gtkstuff import threadslock
 from ..gtkstuff import IconChooserButton
 from ..gtkstuff import IconPreviewFileChooserDialog
 from ..gtkstuff import timeout_add
 
 
-import gettext
 t = gettext.translation(FGlobs.package_name, FGlobs.localedir, fallback=True)
 _ = t.gettext
 
 
-
-gtk.window_set_default_icon_from_file(PGlobs.default_icon)
-
-
-
-class ProfileEntry(gtk.Entry):
+class ProfileEntry(Gtk.Entry):
     _allowed = (65056, 65361, 65363, 65365, 65288, 65289, 65535)
-    
+
     def __init__(self):
-        gtk.Entry.__init__(self)
+        Gtk.Entry.__init__(self)
         self.set_max_length(MAX_PROFILE_LENGTH)
         self.connect("key-press-event", self._cb_kp)
         self.connect("button-press-event", self._cb_button)
-    
-    
+
     def _cb_kp(self, widget, event):
         if not event.keyval in self._allowed and not \
                                     profile_name_valid(event.string):
             return True
-            
-            
+
     def _cb_button(self, widget, event):
         if event.button != 1:
             return True
 
 
+class NewProfileDialog(Gtk.Dialog):
+    _icon_dialog = IconPreviewFileChooserDialog(title="Choose An Icon",
+                                                action=Gtk.FileChooserAction.OPEN)
+    _icon_dialog.add_button(_("Clear"), Gtk.ResponseType.NONE)
+    _icon_dialog.add_button(_("Cancel"), Gtk.ResponseType.CANCEL)
+    _icon_dialog.add_button(_("OK"), Gtk.ResponseType.OK)
 
-class NewProfileDialog(gtk.Dialog):
-    _icon_dialog = IconPreviewFileChooserDialog("Choose An Icon",
-                        buttons = (gtk.STOCK_CLEAR, gtk.RESPONSE_NONE,
-                                      gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
-                                      gtk.STOCK_OK, gtk.RESPONSE_OK))
-    
-    
     def __init__(self, row, filter_function=None, title_extra = "", edit=False):
-        gtk.Dialog.__init__(self)
+        Gtk.Dialog.__init__(self)
         self.set_border_width(6)
         self.get_child().set_spacing(12)
         self.set_modal(True)
         self.set_destroy_with_parent(True)
         self._icon_dialog.set_transient_for(self)
-        
+
         if row is not None:
             if edit:
                 # TC: data entry dialog window title text. %s = profile name
@@ -106,19 +94,19 @@ class NewProfileDialog(gtk.Dialog):
             title = _("New profile details")
         self.set_title(title + title_extra)
 
-        hbox = gtk.HBox()
+        hbox = Gtk.HBox()
         hbox.set_border_width(6)
         hbox.set_spacing(12)
         if edit:
-            icon = gtk.STOCK_EDIT
+            icon_name = "document-edit-symbolic"
         else:
-            icon = gtk.STOCK_COPY if row else gtk.STOCK_NEW
-        self.image = gtk.image_new_from_stock(icon, gtk.ICON_SIZE_DIALOG)
-        self.image.set_alignment(0.0, 0.0)
+            icon_name = "edit-copy-symbolic" if row else "document-new-symbolic"
+        self.image = Gtk.Image.new_from_icon_name(icon_name, Gtk.IconSize.DIALOG)
+        self.image.set_halign(Gtk.Align.START)
+        self.image.set_valign(Gtk.Align.START)
         hbox.pack_start(self.image, False)
-        table = gtk.Table(2, 4)
-        table.set_row_spacings(6)
-        table.set_col_spacing(0, 6)
+        table = Gtk.Grid()
+        table.set_row_spacing(4)
         hbox.pack_start(table)
 
         labels = (
@@ -133,106 +121,89 @@ class NewProfileDialog(gtk.Dialog):
         names = ("profile_entry", "icon_button", "nickname_entry",
                                                         "description_entry")
         widgets = (ProfileEntry(), IconChooserButton(self._icon_dialog),
-                      gtk.Entry(), gtk.Entry())
+                      Gtk.Entry(), Gtk.Entry())
 
         for i, (label, name, widget) in enumerate(zip(labels, names, widgets)):
-            label = gtk.Label(label)
-            label.set_alignment(1.0, 0.5)
-            table.attach(label, 0, 1, i, i + 1, gtk.SHRINK | gtk.FILL)
-
-            table.attach(widget, 1, 2, i, i + 1, yoptions=gtk.SHRINK)
+            label = Gtk.Label.new(label)
+            label.set_xalign(1.0)
+            label.set_margin_end(4)
+            table.attach(label, 0, i, 1, 1)
+            table.attach(widget, 1, i, 1, 1)
+            widget.set_hexpand(True)
             setattr(self, name, widget)
 
         self.profile_entry.set_width_chars(30)
         self.get_content_area().add(hbox)
-        bb = self.get_action_area()
-        bb.set_spacing(6)
 
         if row is not None:
             profile = row[1] if edit else ""
-            revert = gtk.Button(stock=gtk.STOCK_REFRESH)
-            revert.connect("clicked", self._revert, row, edit)
-            revert.clicked()
-            bb.add(revert)
-            bb.set_child_secondary(revert, True)
+
+            profile_text = row[1] if edit else ""
+            self.profile_entry.set_text(profile_text)
+            self.icon_button.set_filename(row[4])
+            self.nickname_entry.set_text(row[5])
+            self.description_entry.set_text(row[2])
+            self.profile_entry.grab_focus()
         else:
             self.icon_button.set_filename(PGlobs.default_icon)
 
         if edit:
             if self.profile_entry.get_text() == default:
                 self.profile_entry.set_sensitive(False)
-            self.delete = gtk.Button(stock=gtk.STOCK_DELETE)
+            self.delete = Gtk.Button(label=_("Delete"))
             self.delete.connect_after("clicked", lambda w: self.destroy())
-            bb.add(self.delete)
-        cancel = gtk.Button(stock=gtk.STOCK_CANCEL)
+            self.add_action_widget(self.delete, Gtk.ResponseType.NONE)
+        cancel = Gtk.Button(label=_("Cancel"))
         cancel.connect("clicked", lambda w: self.destroy())
-        bb.add(cancel)
-        self.ok = gtk.Button(stock=gtk.STOCK_OK)
-        bb.add(self.ok)
-
-
-    def _revert(self, widget, row, edit):
-        profile_text = row[1] if edit else ""
-        self.profile_entry.set_text(profile_text)
-        self.icon_button.set_filename(row[4])
-        self.nickname_entry.set_text(row[5])
-        self.description_entry.set_text(row[2])
-        self.profile_entry.grab_focus()
-
+        self.add_action_widget(cancel, Gtk.ResponseType.NONE)
+        self.ok = Gtk.Button(label=_("OK"))
+        self.add_action_widget(self.ok, Gtk.ResponseType.NONE)
 
     @classmethod
     def append_dialog_title(cls, text):
         cls._icon_dialog.set_title(cls._icon_dialog.get_title() + text)
-        
 
 
-class ProfileSingleton(Singleton, type(gtk.Dialog)):
+class ProfileSingleton(Singleton, type(Gtk.Dialog)):
     def __call__(cls, *args, **kwds):
         return super(ProfileSingleton, cls).__call__(*args, **kwds)
-    
-    
 
-class ProfileDialog(gtk.Dialog):
-    __metaclass__ = ProfileSingleton
-    
-    
-    __gproperties__ = {  "selection-active" : (gobject.TYPE_BOOLEAN, 
-                                "selection active", 
+
+class ProfileDialog(Gtk.Dialog, metaclass=ProfileSingleton):
+    __gproperties__ = {  "selection-active" : (GObject.TYPE_BOOLEAN,
+                                "selection active",
                                 "selected profile is active",
-                                0, gobject.PARAM_READABLE),
-                                
-                                "selection": (str, "profile selection", 
+                                0, GObject.ParamFlags.READABLE),
+
+                                "selection": (str, "profile selection",
                                 "profile selected in profile manager",
                                 "", MAX_PROFILE_LENGTH)
     }
 
     _signal_names = "choose", "delete", "auto"
     _new_profile_dialog_signal_names = "new", "clone", "edit"
-
     __gsignals__ = { "selection-active-changed" : (
-                                gobject.SIGNAL_RUN_FIRST, gobject.TYPE_NONE,
-                                (str, gobject.TYPE_BOOLEAN,)),
-                                
+                                GObject.SignalFlags.RUN_FIRST, GObject.TYPE_NONE,
+                                (str, GObject.TYPE_BOOLEAN,)),
+
                           "selection-changed" : (
-                                gobject.SIGNAL_RUN_FIRST, gobject.TYPE_NONE,
+                                GObject.SignalFlags.RUN_FIRST, GObject.TYPE_NONE,
                                 (str,))
     }
 
     __gsignals__.update(dict(
-            (x, (gobject.SIGNAL_RUN_FIRST, gobject.TYPE_NONE,
-            (gobject.TYPE_STRING,))) for x in (_signal_names)))
+            (x, (GObject.SignalFlags.RUN_FIRST, GObject.TYPE_NONE,
+            (GObject.TYPE_STRING,))) for x in (_signal_names)))
 
     __gsignals__.update(dict(
-            (x, (gobject.SIGNAL_RUN_FIRST, gobject.TYPE_NONE,
-            (gobject.TYPE_STRING,) * 5))
+            (x, (GObject.SignalFlags.RUN_FIRST, GObject.TYPE_NONE,
+            (GObject.TYPE_STRING,) * 5))
             for x in (_new_profile_dialog_signal_names)))
 
-
     @property
     def profile(self):
         return self._profile
 
-
     def __init__(self, default, data_function=None):
         self._default = default
         self._profile = self._highlighted = None
@@ -241,104 +212,101 @@ class ProfileDialog(gtk.Dialog):
         self._title_extra = ""
 
         # TC: profile dialog window title text.
-        gtk.Dialog.__init__(self, _("IDJC Profile Manager"))
-        self.set_size_request(600, 300)
+        Gtk.Dialog.__init__(self, title=_("IDJC Profile Manager"))
         self.set_border_width(6)
-        w = gtk.ScrolledWindow()
-        w.set_border_width(6)
-        w.set_shadow_type(gtk.SHADOW_ETCHED_OUT)
-        w.set_policy(gtk.POLICY_NEVER, gtk.POLICY_AUTOMATIC)
+        w = Gtk.ScrolledWindow()
+        w.set_margin_bottom(5)
+        w.set_size_request(600, 300)
+        w.set_policy(Gtk.PolicyType.NEVER, Gtk.PolicyType.AUTOMATIC)
         self.get_content_area().add(w)
-        self.store = gtk.ListStore(
-                            gtk.gdk.Pixbuf, str, str, int, str, str, int, int)
-        self.sorted = gtk.TreeModelSort(self.store)
+        self.store = Gtk.ListStore(GdkPixbuf.Pixbuf, str, str, int,
+                                   str, str, int, int)
+        self.sorted = Gtk.TreeModelSort(model=self.store)
         self.sorted.set_sort_func(1, self._sort_func)
-        self.sorted.set_sort_column_id(1, gtk.SORT_ASCENDING)
-        self.treeview = gtk.TreeView(self.sorted)
+        self.sorted.set_sort_column_id(1, Gtk.SortType.ASCENDING)
+        self.treeview = Gtk.TreeView(model=self.sorted)
         self.treeview.set_headers_visible(True)
-        self.treeview.set_rules_hint(True)
         w.add(self.treeview)
-        autorend = gtk.CellRendererPixbuf()
-        autorend.props.width = 16
-        autorend.props.stock_id = gtk.STOCK_APPLY
-        autorend.props.stock_size = gtk.ICON_SIZE_MENU
-        pbrend = gtk.CellRendererPixbuf()
+        autorend = Gtk.CellRendererToggle()
+        autorend.set_activatable(True)
+        autorend.connect("toggled", self._cb_autorend_toggle)
+        pbrend = Gtk.CellRendererPixbuf()
         pbrend.props.width = 16
-        strrend = gtk.CellRendererText()
+        strrend = Gtk.CellRendererText()
         ledrend = CellRendererLED()
         time_rend = CellRendererTime()
-        strrend_ellip = gtk.CellRendererText()
-        strrend_ellip.props.ellipsize = pango.ELLIPSIZE_END
+        strrend_ellip = Gtk.CellRendererText()
+        strrend_ellip.props.ellipsize = Pango.EllipsizeMode.END
         # TC: column heading. The available profile names appears below.
-        c0 = gtk.TreeViewColumn(None, autorend, visible=7)
-        image = gtk.image_new_from_stock(gtk.STOCK_OPEN, gtk.ICON_SIZE_MENU)
+        c0 = Gtk.TreeViewColumn(None, autorend, active=7)
+        image = Gtk.Image.new_from_icon_name("document-open-symbolic", Gtk.IconSize.MENU)
         c0.set_widget(image)
         image.show()
         self.treeview.append_column(c0)
-        c1 = gtk.TreeViewColumn(_("Profile"))
+        c1 = Gtk.TreeViewColumn(_("Profile"))
         c1.pack_start(pbrend, expand=False)
-        c1.pack_start(strrend)
+        c1.pack_start(strrend, expand=True)
         c1.add_attribute(pbrend, "pixbuf", 0)
         c1.add_attribute(strrend, "text", 1)
         c1.set_spacing(2)
         self.treeview.append_column(c1)
         # TC: column heading. The profile nicknames.
-        c2 = gtk.TreeViewColumn(_("Nickname"), strrend, text=5)
+        c2 = Gtk.TreeViewColumn(_("Nickname"), strrend, text=5)
         self.treeview.append_column(c2)
         # TC: column heading.
-        c3 = gtk.TreeViewColumn(_("Description"), strrend_ellip, text=2)
+        c3 = Gtk.TreeViewColumn(_("Description"), strrend_ellip, text=2)
         c3.set_expand(True)
         self.treeview.append_column(c3)
         # TC: column heading. The time a particular profile has been running.
-        c4 = gtk.TreeViewColumn(_("Up-time"))
-        c4.pack_start(ledrend)
-        c4.pack_start(time_rend)
+        c4 = Gtk.TreeViewColumn(_("Up-time"))
+        c4.pack_start(ledrend, expand=True)
+        c4.pack_start(time_rend, expand=True)
         c4.add_attribute(ledrend, "active", 3)
         c4.add_attribute(time_rend, "time", 6)
         c4.set_spacing(2)
         self.treeview.append_column(c4)
         self.selection = self.treeview.get_selection()
         self.selection.connect("changed", self._cb_selection)
-        box = self.get_action_area()
+        box = Gtk.ButtonBox()
         box.set_spacing(6)
-        for attr, label, sec, stock in zip(
-                ("new", "clone", "edit", "delete", "auto", "cancel", "choose"), 
-                (gtk.STOCK_NEW, gtk.STOCK_COPY, gtk.STOCK_EDIT,
-                 gtk.STOCK_DELETE, _("_Auto"), gtk.STOCK_QUIT, gtk.STOCK_OPEN),
-                (True,) * 4 + (False,) * 3,
-                (True,) * 4 + (False,) + (True,) * 2):
-            w = gtk.Button(label)
-            w.set_use_stock(stock)
+        box.set_layout(Gtk.ButtonBoxStyle.END)
+        for attr, icon_name, sec in zip(
+                ("new", "clone", "edit", "delete", "cancel", "choose"),
+                ("document-new-symbolic",
+                 "edit-copy-symbolic",
+                 "document-edit-symbolic",
+                 "document-delete-symbolic",
+                 "window-close-symbolic",
+                 "system-run-symbolic"),
+                (True,) * 4 + (False,) * 2):
+            w = Gtk.Button.new_from_icon_name(icon_name, Gtk.IconSize.BUTTON)
             box.add(w)
             box.set_child_secondary(w, sec)
             setattr(self, attr, w)
+        self.get_content_area().add(box)
 
         self.delete.set_no_show_all(True)
         self.cancel.connect("clicked", self._cb_cancel)
         self.set_data_function(data_function)
         self.connect("notify::visible", self._cb_visible)
-        for each in self._signal_names:
+        for each in "choose", "delete":
             getattr(self, each).connect("clicked", self._cb_click, each)
         for each in self._new_profile_dialog_signal_names:
             getattr(self, each).connect("clicked", self._cb_new_profile_dialog,
                                                                         each)
-     
-    
+
     def display_error(self, message, transient_parent=None, markup=False):
         error_dialog = ErrorMessageDialog("", message, markup=markup)
         error_dialog.set_transient_for(transient_parent or self)
         error_dialog.show_all()
 
-    
     def destroy_new_profile_dialog(self):
         self._new_profile_dialog.destroy()
         del self._new_profile_dialog
 
-
     def get_new_profile_dialog(self):
         return self._new_profile_dialog
-    
-    
+
     def do_get_property(self, prop):
         if prop.name == "selection-active":
             return self._selection_active
@@ -346,7 +314,6 @@ class ProfileDialog(gtk.Dialog):
             return self._highlighted
         else:
             raise AttributeError("unknown property: %s" % prop.name)
-    
 
     def do_selection_active_changed(self, profile, state):
         state = not state
@@ -354,7 +321,9 @@ class ProfileDialog(gtk.Dialog):
         self.edit.set_sensitive(state)
         self.clone.set_sensitive(state)
 
-    
+    def _cb_autorend_toggle(self, *args):
+        self.emit("auto", self._highlighted)
+
     def _cb_click(self, widget, signal):
         if self._highlighted is not None:
             def commands():
@@ -365,11 +334,11 @@ class ProfileDialog(gtk.Dialog):
                 if self._highlighted == self._default:
                     message = _("<span weight='bold' size='12000'>Delete the"
                             " data of profile '%s'?</span>\n\nThe profile will"
-                            " remain available with initial settings.")
+                            " be recreated with initial settings.")
                 else:
                     message = _("<span weight='bold' size='12000'>Delete "
-                            "profile '%s' and all its data?</span>\n\nThe"
-                            " data of deleted profiles cannot be recovered.")
+                            "profile '%s' and all of its data?</span>\n\nThis"
+                            " action cannot be undone.")
                 conf = ConfirmationDialog("", message % self._highlighted,
                                                                 markup=True)
                 conf.set_transient_for(self)
@@ -378,7 +347,6 @@ class ProfileDialog(gtk.Dialog):
             else:
                 commands()
 
-    
     def _cb_new_profile_dialog(self, widget, action):
         if action in ("clone", "edit"):
             if self._highlighted is None:
@@ -388,11 +356,11 @@ class ProfileDialog(gtk.Dialog):
         else:
             row = None
             template = None
-            
+
         np_dialog = self._new_profile_dialog = NewProfileDialog(row,
                     title_extra = self._title_extra, edit=action=="edit")
         np_dialog.set_transient_for(self)
-        
+
         def sub_ok(widget):
             profile = np_dialog.profile_entry.get_text()
             icon = np_dialog.icon_button.get_filename()
@@ -401,31 +369,27 @@ class ProfileDialog(gtk.Dialog):
             self.emit(action, profile, template, icon, nickname, description)
             self._update_data()
             self.highlight_profile(profile)
-            
+
         np_dialog.ok.connect("clicked", sub_ok)
         if action == "edit":
             np_dialog.delete.connect("clicked", lambda w: self.delete.clicked())
         np_dialog.show_all()
 
-
     def _cb_cancel(self, widget):
         if self._profile is None:
             self.response(0)
         else:
             self.hide()
 
-
     def _cb_delete_event(self, widget, event):
         self.hide()
         return True
 
-    
     def _cb_visible(self, *args):
         self._update_data()
         if self.props.visible:
-            timeout_add(200, threadslock(self._update_data))
-        
-        
+            timeout_add(200, self._update_data)
+
     def _cb_selection(self, ts):
         model, iter = ts.get_selected()
         if iter is not None:
@@ -440,23 +404,22 @@ class ProfileDialog(gtk.Dialog):
         if active != self._selection_active:
             self._selection_active = active
             self.emit("selection-active-changed", self._highlighted, active)
-                
-        
+
     def highlight_profile(self, target, scroll=True):
         i = self._get_index_for_profile(target)
+        if i is None:
+            i = self._get_index_for_profile("default")
         if i is not None:
             self.selection.select_path(i)
             if scroll:
                 self.selection.get_tree_view().scroll_to_cell(i)
 
-
     def _get_index_for_profile(self, target):
         for i, data in enumerate(self.sorted):
             if data[1] == target:
                 return i
         return None
 
-
     def _get_row_for_profile(self, target):
         path = self._get_index_for_profile(target)
         if path is not None:
@@ -464,15 +427,13 @@ class ProfileDialog(gtk.Dialog):
         else:
             return None
 
-
     def _sort_func(self, model, *iters):
-        vals = tuple(model.get_value(x, 1) for x in iters)
-        
+        vals = tuple(model.get_value(x, 1) for x in iters if x is not None)
+
         try:
             return vals.index(self._default)
         except ValueError:
             return cmp(*vals)
-        
 
     def set_data_function(self, f):
         self._data_function = f
@@ -480,14 +441,9 @@ class ProfileDialog(gtk.Dialog):
         if f is not None:
             self.highlight_profile(self._default)
 
-
     def _auto_data_function(self, col, cell, model, iter):
         val = model.get_value(iter, 7)
-        cell.set_visible(val)
-        if val:
-            cell.props.stock_id = gtk.STOCK_APPLY
-            cell.props.stock_size = gtk.ICON_SIZE_MENU
-
+        cell.set_active(val)
 
     def _update_data(self):
         if self._data_function is not None:
@@ -508,8 +464,8 @@ class ProfileDialog(gtk.Dialog):
                             i = None
                     if i is not None:
                         try:
-                            pb = gtk.gdk.pixbuf_new_from_file_at_size(i, 16, 16)
-                        except glib.GError:
+                            pb = GdkPixbuf.Pixbuf.new_from_file_at_size(i, 16, 16)
+                        except GLib.GError:
                             pb = i = None
                     else:
                         pb = None
@@ -523,8 +479,7 @@ class ProfileDialog(gtk.Dialog):
                 self.selection.handler_unblock_by_func(self._cb_selection)
                 self.highlight_profile(h, scroll=False)
         return self.props.visible
-        
-        
+
     def set_profile(self, newprofile, title_extra, iconpathname):
         assert self._profile is None
         self.hide()
@@ -535,23 +490,23 @@ class ProfileDialog(gtk.Dialog):
         self._title_extra = title_extra
         try:
             self.set_icon_from_file(iconpathname)
-        except glib.GError:
+        except GLib.GError:
             print("Profile icon image file not found:", iconpathname)
         else:
-            gtk.window_set_default_icon_from_file(iconpathname)
+            Gtk.Window.set_default_icon_from_file(iconpathname)
 
-        self.cancel.set_label(gtk.STOCK_CLOSE)
         self.connect("delete-event", self._cb_delete_event)
         self.response(0)
-        
-        
+
     def run(self):
         if self._profile is None:
             self.show_all()
-            gtk.Dialog.run(self)
+            try:
+                Gtk.Dialog.run(self)
+            except KeyboardInterrupt:
+                pass
         else:
             self.show()
 
-
     def present(self):
         self.show_all()
